module BusinessInsightApiClient

  # This module contains all errors that can be thrown by making requests to the API.
  # All errors inherit from {StandardError}.
  #
  # @see Errors::BusinessInsightApiRequestError
  # @see Errors::BadRequestError
  # @see Errors::UnauthorizedError
  # @see Errors::ForbiddenError
  # @see Errors::NotFoundError
  # @see Errors::InternalError
  # @see Errors::UnavailableError
  # @see Errors::RequestTimedOutError
  module Errors

    # General error class for Request based errors.
    class BusinessInsightApiRequestError < StandardError

      # @return [String] the full error message as received from the server.
      attr_reader :full_code

      # @return [Integer] the status code as received from the server
      attr_reader :status_code

      def initialize(status_code, full_code)
        @status_code = status_code
        @full_code = full_code
      end
    end

    # This error is thrown when a bad request is made.
    # This is a 400 status error.
    class BadRequestError         < BusinessInsightApiRequestError; end

    # This error is thrown when a unauthorized request was made by the client.
    # This is mostly caused by a missing or expired token.
    # This is a 401 status error.
    class UnauthorizedError       < BusinessInsightApiRequestError; end

    # This error is thrown when a request is made with insufficient scopes.
    # This is an 403 status error.
    class ForbiddenError          < BusinessInsightApiRequestError; end

    # This error is thrown when the page to which the request was made was not found.
    # This is a 404 status error.
    class NotFoundError           < BusinessInsightApiRequestError; end

    # This error is thrown when the request was malformed and the server had trouble processing the request.
    # Possible due internal server bugs.
    # This is a 500 status error.
    class InternalError           < BusinessInsightApiRequestError; end

    #501, 502, 503
    class UnavailableError        < BusinessInsightApiRequestError; end

    # This error is thrown when the request made was timed out.
    # This could be because the server is unavailable to request. Check your internet settings or try again later.
    class RequestTimedOutError    < StandardError; end
  end
end