module BusinessInsightApiClient
  module Helpers
    require 'uri'

    # Authorization helper, used as helper to interact with the OAuth2 authorization server.
    class Authorization

      DEFAULT_OPTIONS = {
          authorization_url: 'https://nedap-bi.com',
          client_id: '',
          client_secret: ''
      }


      # @return [OAuth2:AccessToken] the access token.
      attr_accessor :current_token

      # @return [URI] the uri of the authorization server
      attr_reader :authorization_url

      # @return [String] the client id
      attr_reader :client_id

      # @return [String] the client secret
      attr_reader :client_secret

      # Creates a new authorization helper
      #
      # @param [Hash] options the options to create the Authorization Helper with.
      # @option options [String] :authorization_url ('https://nedap-bi.com') the authorization url.
      # @option options [String] :client_id ('') the client id
      # @option options [String] :client_secret ('') the client secret
      # @raise [URI::InvalidURIError] when the authorization_url given can not be parsed into an uri.
      def initialize(options = {})
        @authorization_url = URI(options[:authorization_url] || DEFAULT_OPTIONS[:authorization_url])
        @client_id = options[:client_id] || DEFAULT_OPTIONS[:client_id]
        @client_secret = options[:client_secret] || DEFAULT_OPTIONS[:client_secret]
      end

      # @return [OAuth2::AccessToken] returns the client credential token for the application.
      def client_credential_token
        oauth_client.client_credentials.get_token
      end


      # @return [Hash] the headers hash (includes Authorization token) as formatted with the configuration
      def current_token_headers
        return {} if current_token.nil?
        current_token.headers
      end

      # Creates an URL to the authorization endpoint of the provider.
      # When the user authorizes he will direct the user agent that follows this link to the redirect_uri.
      #
      # @param [Hash] params additional query parameters for the URL.
      # @option params [String] :redirect_uri  the redirect uri to which the client has to be redirected to.
      def auth_code_authorize_url(params = {})
        oauth_client.auth_code.authorize_url(params)
      end

      # Retrieve an access token given the specified validation code.
      # When the token is verified it will direct the user agent that follows this link to the redirect_uri.
      #
      # @param [String] access_grant The Authorization Code value (Access Grant)
      # @param [Hash] params additional params
      # @option params [String] :redirect_uri  the redirect uri to which the client has to be redirected to.
      # @param [Hash] options options
      def auth_code_token_from_access_grant(access_grant, params={}, options={})
        oauth_client.auth_code.get_token(access_grant, params, options)
      end


      # Creates an AccessToken from hash
      #
      # @param [Hash] options the options to create the Access Token with
      # @option options [String] :token token the Access Token value
      # @option options [String] :refresh_token (nil) the refresh_token value
      # @option options [FixNum, String] :expires_in (nil) the number of seconds in which the AccessToken will expire
      # @option options [FixNum, String] :expires_at (nil) the epoch time in seconds in which AccessToken will expire
      # @return [OAuth2::AccessToken] an access token that can be used for API requests.
      # @see http://www.rubydoc.info/gems/oauth2/1.0.0/OAuth2/AccessToken OAuth2::AccessToken information
      def access_token_from_hash(options = {})
        OAuth2::AccessToken.new oauth_client, options.delete(:token), options
      end

      private

      def oauth_client
        # raise_errors: false, to suppress OAuth2::Errors and to allow custom error handling
        @oauth_client ||= OAuth2::Client.new(@client_id, @client_secret, site: @authorization_url, raise_errors: false)
      end
    end
  end
end
